@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 2, 7)
+__version__ = (0, 2, 8)
 __all__ = [
     "SupportsRead", "SupportsReadinto", "SupportsWrite", "SupportsSeek", 
     "AsyncBufferedReader", "AsyncTextIOWrapper", "buffer_length", 
@@ -11,7 +11,7 @@ __all__ = [
     "bytes_iter_to_reader", "bytes_iter_to_async_reader", "bytes_to_chunk_iter", 
     "bytes_to_chunk_async_iter", "bytes_ensure_part_iter", "bytes_ensure_part_async_iter", 
     "progress_bytes_iter", "progress_bytes_async_iter", "copyfileobj", "copyfileobj_async", 
-    "bound_bufferd_reader", "bound_bufferd_async_reader", 
+    "bound_bytes_reader", "bound_async_bytes_reader", 
 ]
 
 from asyncio import Lock as AsyncLock
@@ -31,8 +31,7 @@ from asynctools import async_chain, ensure_async, ensure_aiter, run_async
 from property import funcproperty
 
 
-READ_BUFSIZE = 1 << 13
-WRITE_BUFSIZE = 1 << 16
+READ_BUFSIZE = 1 << 16
 CRE_NOT_UNIX_NEWLINES: Final = re_compile("\r\n|\r")
 
 
@@ -46,7 +45,7 @@ class VirtualBufferedReader:
 
 @runtime_checkable
 class SupportsRead[T](Protocol):
-    def read(self, /, size: int) -> T: ...
+    def read(self, /, size: int = -1) -> T: ...
 
 
 @runtime_checkable
@@ -827,17 +826,16 @@ def get_bom(encoding: str, /) -> bytes:
 
 
 def buffer_length(b: Buffer, /) -> int:
-    try:
-        return len(b) # type: ignore
-    except:
-        return len(memoryview(b))
+    if isinstance(b, (bytes, bytearray, memoryview)):
+        return len(b)
+    return len(memoryview(b))
 
 
 def bio_chunk_iter(
     bio: SupportsRead[Buffer] | SupportsReadinto | Callable[[int], Buffer], 
     /, 
     size: int = -1, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
     can_buffer: bool = False, 
     callback: None | Callable[[int], Any] = None, 
 ) -> Iterator[Buffer]:
@@ -906,7 +904,7 @@ async def bio_chunk_async_iter(
     bio: SupportsRead[Buffer] | SupportsRead[Awaitable[Buffer]] | SupportsReadinto | Callable[[int], Buffer] | Callable[[int], Awaitable[Buffer]], 
     /, 
     size: int = -1, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
     can_buffer: bool = False, 
     callback: None | Callable[[int], Any] = None, 
 ) -> AsyncIterator[Buffer]:
@@ -974,7 +972,7 @@ def bio_skip_iter(
     bio: SupportsRead[Buffer] | SupportsReadinto | Callable[[int], Buffer], 
     /, 
     size: int = -1, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
     callback: None | Callable[[int], Any] = None, 
 ) -> Iterator[int]:
     if size == 0:
@@ -990,7 +988,7 @@ def bio_skip_iter(
             length = seek(0, 2) - curpos
     except Exception:
         if chunksize <= 0:
-            chunksize = WRITE_BUFSIZE
+            chunksize = READ_BUFSIZE
         if callable(bio):
             read = bio
         elif hasattr(bio, "readinto"):
@@ -1045,7 +1043,7 @@ async def bio_skip_async_iter(
     bio: SupportsRead[Buffer] | SupportsRead[Awaitable[Buffer]] | SupportsReadinto | Callable[[int], Buffer] | Callable[[int], Awaitable[Buffer]], 
     /, 
     size: int = -1, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
     callback: None | Callable[[int], Any] = None, 
 ) -> AsyncIterator[int]:
     if size == 0:
@@ -1061,7 +1059,7 @@ async def bio_skip_async_iter(
             length = (await seek(0, 2)) - curpos
     except Exception:
         if chunksize <= 0:
-            chunksize = WRITE_BUFSIZE
+            chunksize = READ_BUFSIZE
         if callable(bio):
             read: Callable[[int], Awaitable[Buffer]] = ensure_async(bio, threaded=True)
         elif hasattr(bio, "readinto"):
@@ -1275,28 +1273,28 @@ def bytes_iter_to_reader(
         if at_end or not (bufsize := buffer_length(buf)):
             return 0
         with lock:
-            n = buffer_length(unconsumed)
-            if bufsize <= n:
-                buf[:], unconsumed = unconsumed[:bufsize], unconsumed[bufsize:]
-                pos += bufsize
-                return bufsize
-            buf[:n] = unconsumed
-            del unconsumed[:]
+            if n := buffer_length(unconsumed):
+                if bufsize <= n:
+                    buf[:], unconsumed = unconsumed[:bufsize], unconsumed[bufsize:]
+                    pos += bufsize
+                    return bufsize
+                buf[:n] = unconsumed
+                pos += n
+                del unconsumed[:]
             try:
                 while True:
-                    b = memoryview(getnext())
-                    if not b:
-                        continue
-                    m = n + buffer_length(b)
-                    if m >= bufsize:
-                        buf[n:] = b[:bufsize-n]
-                        unconsumed += b[bufsize-n:]
-                        pos += bufsize
-                        return bufsize
-                    else:
-                        buf[n:m] = b
-                        pos += buffer_length(b)
-                        n = m
+                    if b := memoryview(getnext()):
+                        m = n + len(b)
+                        if m >= bufsize:
+                            delta = bufsize - n
+                            buf[n:] = b[:delta]
+                            unconsumed += b[delta:]
+                            pos += delta
+                            return bufsize
+                        else:
+                            buf[n:m] = b
+                            pos += len(b)
+                            n = m
             except StopIteration:
                 at_end = True
                 return n
@@ -1401,15 +1399,7 @@ def bytes_iter_to_async_reader(
         except:
             pass
     def close():
-        nonlocal at_end
-        try:
-            method = getattr(it, "aclose")
-        except AttributeError:
-            method = getattr(it, "close")
-        ret = method()
-        if isawaitable(ret):
-            run_async(ret)
-        at_end = True
+        run_async(aclose())
     async def aclose():
         nonlocal at_end
         try:
@@ -1452,28 +1442,28 @@ def bytes_iter_to_async_reader(
         if at_end or not (bufsize := buffer_length(buf)):
             return 0
         async with lock:
-            n = buffer_length(unconsumed)
-            if bufsize <= n:
-                buf[:], unconsumed = unconsumed[:bufsize], unconsumed[bufsize:]
-                pos += bufsize
-                return bufsize
-            buf[:n] = unconsumed
-            del unconsumed[:]
+            if n := buffer_length(unconsumed):
+                if bufsize <= n:
+                    buf[:], unconsumed = unconsumed[:bufsize], unconsumed[bufsize:]
+                    pos += bufsize
+                    return bufsize
+                buf[:n] = unconsumed
+                pos += n
+                del unconsumed[:]
             try:
                 while True:
-                    b = memoryview(await getnext())
-                    if not b:
-                        continue
-                    m = n + buffer_length(b)
-                    if m >= bufsize:
-                        buf[n:] = b[:bufsize-n]
-                        unconsumed += b[bufsize-n:]
-                        pos += bufsize
-                        return bufsize
-                    else:
-                        buf[n:m] = b
-                        pos += buffer_length(b)
-                        n = m
+                    if b := memoryview(await getnext()):
+                        m = n + len(b)
+                        if m >= bufsize:
+                            delta = bufsize - n
+                            buf[n:] = b[:delta]
+                            unconsumed += b[delta:]
+                            pos += delta
+                            return bufsize
+                        else:
+                            buf[n:m] = b
+                            pos += len(b)
+                            n = m
             except (StopIteration, StopAsyncIteration):
                 at_end = True
                 return n
@@ -1566,7 +1556,7 @@ def bytes_iter_to_async_reader(
 def bytes_to_chunk_iter(
     b: Buffer, 
     /, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
 ) -> Iterator[memoryview]:
     m = memoryview(b)
     for i in range(0, buffer_length(m), chunksize):
@@ -1576,7 +1566,7 @@ def bytes_to_chunk_iter(
 async def bytes_to_chunk_async_iter(
     b: Buffer, 
     /, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
 ) -> AsyncIterator[memoryview]:
     m = memoryview(b)
     for i in range(0, buffer_length(m), chunksize):
@@ -1586,7 +1576,7 @@ async def bytes_to_chunk_async_iter(
 def bytes_ensure_part_iter(
     it: Iterable[Buffer], 
     /, 
-    partsize: int = WRITE_BUFSIZE, 
+    partsize: int = READ_BUFSIZE, 
 ) -> Iterator[Buffer]:
     n = partsize
     for b in it:
@@ -1614,7 +1604,7 @@ def bytes_ensure_part_iter(
 async def bytes_ensure_part_async_iter(
     it: Iterable[Buffer] | AsyncIterable[Buffer], 
     /, 
-    partsize: int = WRITE_BUFSIZE, 
+    partsize: int = READ_BUFSIZE, 
 ) -> AsyncIterator[Buffer]:
     n = partsize
     async for b in ensure_aiter(it):
@@ -1723,10 +1713,10 @@ def copyfileobj(
     fsrc, 
     fdst: SupportsWrite[Buffer], 
     /, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
 ):
     if chunksize <= 0:
-        chunksize = WRITE_BUFSIZE
+        chunksize = READ_BUFSIZE
     fdst_write = fdst.write
     fsrc_read = getattr(fsrc, "read", None)
     fsrc_readinto = getattr(fsrc, "readinto", None)
@@ -1748,11 +1738,11 @@ async def copyfileobj_async(
     fsrc, 
     fdst: SupportsWrite[Buffer], 
     /, 
-    chunksize: int = WRITE_BUFSIZE, 
+    chunksize: int = READ_BUFSIZE, 
     threaded: bool = True, 
 ):
     if chunksize <= 0:
-        chunksize = WRITE_BUFSIZE
+        chunksize = READ_BUFSIZE
     fdst_write = ensure_async(fdst.write, threaded=threaded)
     fsrc_read = getattr(fsrc, "read", None)
     fsrc_readinto = getattr(fsrc, "readinto", None)
@@ -1773,7 +1763,7 @@ async def copyfileobj_async(
                 await fdst_write(chunk)
 
 
-def bound_bufferd_reader(
+def bound_bytes_reader(
     file: SupportsRead[Buffer], 
     size: int = -1, 
 ) -> SupportsRead[Buffer]:
@@ -1807,7 +1797,7 @@ def bound_bufferd_reader(
     return Reader()
 
 
-def bound_bufferd_async_reader(
+def bound_async_bytes_reader(
     file: SupportsRead[Buffer] | SupportsRead[Awaitable[Buffer]], 
     size: int = -1, 
 ) -> SupportsRead[Awaitable[Buffer]]:
